/* This code implements the Wave example from the RenderMan Companion.
*  Most of the variables are handled by the Quadric super class. The
*  only new variable introduced is "nwaves", the number of waves.
*/

#import "3dreality_api.h"
#import "Wave.h"

@implementation Wave

// Most of these methods were lifted from the Sphere bundle example and
// (slightly) modified.

static int shapeNumber = 1;
- (int)shapeNumber
{
    return shapeNumber++;
}

- calcBoundingBox
{
// xmin & xmax
    boundingBox[0]=-radius;
    boundingBox[1]=radius;
    
// ymin & ymax
    boundingBox[2]=-radius; 
    boundingBox[3]=radius;  
    
// zmin & zmax
    boundingBox[4]=-radius/(4.0*nwaves+1);
    boundingBox[5]=radius/(4.0*nwaves+1);
    return self;
}

- init
{
    [super init];
    nwaves = 4;
    radius = 1.0;
    thetaMax = 360.;
    [self calcBoundingBox];
    return self;
}

// New methods for this class
- setnWave:(RtInt *)aValue
{
    nwaves = aValue?*aValue:1;
    [self calcBoundingBox];
    return self;
}

- (RtInt)nWaves
{
	return nwaves;
}

/****
* Create a set of concentric "waves" using a set of semi-tori.
* Number of waves given by 'nwaves'; sweep of the waves given by 'thetamax'
* Taken from the RenderMan Companion (pg. 67) by Steve Upstill. Note that
* there is a slight error (typo) in the book (in the arguments passed to
* RiSphere). Also I have generalized the code to handle a varying radius 
* (the book example is for a fixed radius of one.)
****/
void
TorusWave(nwaves,thetamax,rad)
int nwaves;
float thetamax,rad;
{
	float innerrad, outerrad;
	int wave;

	if(nwaves<1){
		printf("Need positve number of waves\n");
		return;
		}

	innerrad = rad/(4.0*nwaves+1);
	RiRotate(90.0,1.0,0.0,0.0);

	RiSphere(innerrad,-innerrad, 0.0,thetamax,RI_NULL);

	outerrad=0;
	for(wave=1;wave<=nwaves; wave++){
		outerrad = outerrad + (innerrad*2);
		RiTorus(outerrad,innerrad,0.0,180.0,thetamax,RI_NULL);
		outerrad=outerrad + (innerrad*2);
		RiTorus(outerrad,innerrad,180.0,360.0,thetamax,RI_NULL);
		}
}


- doRenderSelf: (Camera *) camera
{
    if(![super doRenderSelf:camera]) return nil;

	TorusWave(nwaves,thetaMax,radius);    

    return self;
}

////////////////////////////////////////////////////////////////////
//			Archive
///////////////////////////////////////////////////////////////////

- read:(NXTypedStream *)stream
{
    [super read:stream];
    NXReadTypes(stream,"i",&nwaves);
    return self;
}


- write:(NXTypedStream *)stream
{
    [super write:stream];
    NXWriteTypes(stream,"i",&nwaves);
    return self;
}

@end
